import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:google_sign_in/google_sign_in.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:flutter_facebook_auth/flutter_facebook_auth.dart';
import 'ProgressDialog.dart';
import 'package:linkedin_login/linkedin_login.dart';
import 'package:sign_in_with_apple/sign_in_with_apple.dart';
import 'dart:math';
import 'dart:convert';
import 'package:crypto/crypto.dart';

class SocialLoginUtils{

  GoogleSignInAccount? googleSignInAccount;
  GoogleSignIn _googleSignIn = GoogleSignIn();


  Future initializeApp()async{
    await Firebase.initializeApp();
  }
  Future<bool> signOut()async{
   return await FirebaseAuth.instance.signOut().then((value) {
      new GoogleSignIn().signOut();
      print("google logout successful");
      return true;
    }).catchError((onError){
      print("google logout onError"+onError.toString());
      return false;
    });
    print("signout return");
    return false;
  }
  Future<User?> signInWithGoogle(BuildContext context,
      {Color? progressBarColors, bool? isDisplayProgressBar}) async {
    ProgressDialog.show(context,progressBarColors??Colors.black,isDisplayProgressBar??false);
    // await signOut();
    if (googleSignInAccount != null) {
      await _googleSignIn.disconnect();
      await FirebaseAuth.instance.signOut();
    }

    googleSignInAccount = await _googleSignIn.signIn();

    if(googleSignInAccount!=null){
      GoogleSignInAuthentication googleSignInAuthentication =
      await googleSignInAccount!.authentication;
      AuthCredential credential = GoogleAuthProvider.credential(
        accessToken: googleSignInAuthentication.accessToken,
        idToken: googleSignInAuthentication.idToken,
      );
     return await FirebaseAuth.instance.signInWithCredential(credential).then((UserCredential result){
        // Map userData=new Map();
        ProgressDialog.hide();

        // userData["email"]=result.user?.email;
        // userData["displayName"]=result.user?.displayName;
        // userData["uid"]=result.user?.uid;
        print("userData="+result.user.toString());
        // loginAPI(context,result.user.email,"",LoginType.google.name,result.user.uid,result.user.displayName);
        print("return userData main");
        return result.user;
      }).catchError((e){
        print(e.toString());
        String errorMessage;
        switch (e.code) {
          case "ERROR_INVALID_EMAIL":
            errorMessage = "Your email address appears to be malformed.";
            break;
          case "ERROR_WRONG_PASSWORD":
            errorMessage = "Your password is wrong.";
            break;
          case "ERROR_USER_NOT_FOUND":
            errorMessage = "User with this email doesn't exist.";
            break;
          case "ERROR_USER_DISABLED":
            errorMessage = "User with this email has been disabled.";
            break;
          case "ERROR_TOO_MANY_REQUESTS":
            errorMessage = "Too many requests. Try again later.";
            break;
          case "ERROR_OPERATION_NOT_ALLOWED":
            errorMessage = "Signing in with Email and Password is not enabled.";
            break;
          default:
            errorMessage = "An undefined Error happened.";
        }
        if(errorMessage!=null){
          showToastMessage(errorMessage);
        }

        ProgressDialog.hide();
        print("return userData error");
        return null;
      });
    }else{
      ProgressDialog.hide();
      // showToastMessage("Something went wrong");
      print("return userData else");
      return null;
      /* Future.delayed(Duration(seconds: 1)).then((valuess) {
        ProgressDialog.hide();
      });*/
    }
    print("return userData end");
    // return user;
  }

  Future<User?> signInWithFacebook(BuildContext context,
      {Color? progressBarColors, bool? isDisplayProgressBar}) async {
    ProgressDialog.show(context,progressBarColors??Colors.black,isDisplayProgressBar??false);
    // Trigger the sign-in flow
    FacebookAuth.instance.logOut();
    FirebaseAuth.instance.signOut();
    try {
      final LoginResult loginResult = await FacebookAuth.instance.login(
          permissions: ["email", "public_profile"],
          loginBehavior: LoginBehavior.webOnly);
      print(loginResult.message);
      // Create a credential from the access token
      if (loginResult != null) {
        final OAuthCredential facebookAuthCredential =
        FacebookAuthProvider.credential(loginResult.accessToken!.token);
        ProgressDialog.hide();
        print(facebookAuthCredential);

        UserCredential result = await FirebaseAuth.instance
            .signInWithCredential(facebookAuthCredential);
        print("user="+result.user.toString());
        return result.user;
      }
    } on FirebaseAuthException catch (e) {
      ProgressDialog.hide();
      showToastMessage(e.message.toString());
      return null;
    } catch (e) {
      ProgressDialog.hide();
      print(e.toString());
      return null;
    }

    return null;
  }

  Future<Map<String, dynamic>?> signInWithLinkedIn(BuildContext context,
      {Color? progressBarColors, bool? isDisplayProgressBar,
        required String redirectUrl,
        required String clientId,
        required String clientSecret,
        required PreferredSizeWidget? appBar,
        bool? destroySession,
      }) async {
    ProgressDialog.show(context,progressBarColors??Colors.black,isDisplayProgressBar??false);
    // Trigger the sign-in flow
    try {
      ProgressDialog.hide();
      return Navigator.push(
        context,
        MaterialPageRoute(
          builder: (BuildContext context) => LinkedInUserWidget(
            appBar: appBar,
            destroySession: destroySession ?? true,
            // redirectUrl: FirebaseKey.redirectUrl,
            redirectUrl: redirectUrl,
            clientId: clientId,
            clientSecret: clientSecret,
            projection: [
              ProjectionParameters.id,
              ProjectionParameters.localizedFirstName,
              ProjectionParameters.localizedLastName,
              ProjectionParameters.firstName,
              ProjectionParameters.lastName,
              ProjectionParameters.profilePicture,
            ],
            onError: (UserFailedAction e) {

              showToastMessage(e.toString());
              print('Error: ${e.toString()}');
              print('Error: ${e.stackTrace.toString()}');
            },
            onGetUserProfile: (UserSucceededAction linkedInUser) {
              print(
                  'Access token ${linkedInUser.user.token.accessToken}');

              print('User id: ${linkedInUser.user.userId}');
              // print('User firstName: ${linkedInUser?.user?.firstName?.localized?.label}');
              // String name=
             /* loginAPI(context,linkedInUser?.user?.email?.elements[0]
                  ?.handleDeep?.emailAddress,"",LoginType.linkedin.name,linkedInUser.user.userId,
                  linkedInUser?.user?.firstName?.localized?.label);*/
              Map<String,dynamic> user=new Map();
              user["firstName"]=linkedInUser?.user?.firstName?.localized?.label;
              user["lastName"]=linkedInUser?.user?.lastName?.localized?.label;
              user["email"]=linkedInUser?.user?.email?.elements?[0]?.handleDeep?.emailAddress;
              user["profileImageUrl"]=linkedInUser?.user?.profilePicture?.displayImageContent
                  ?.elements?[0]?.identifiers?[0]?.identifier;


              /*setState(() {
                                        logoutUser = false;
                                      });*/

              Navigator.pop(context,user);
            },
          ),
          fullscreenDialog: true,
        ),
      );
    } catch (e) {
      ProgressDialog.hide();
      print(e.toString());
      return null;
    }

    return null;
  }

  Future<User?> appleLogin(BuildContext context,bool? isAndroid,
      {Color? progressBarColors, bool? isDisplayProgressBar,String? clientID,}) async {
    ProgressDialog.show(context,progressBarColors??Colors.black,isDisplayProgressBar??false);
    try {
      late AuthorizationCredentialAppleID appleCredential;
      final rawNonce = generateNonce();
      final nonce = sha256ofString(rawNonce);
      // Request credential for the currently signed in Apple account.
    if(isAndroid!=null && isAndroid){
    var redirectURL = "https://SERVER_AS_PER_THE_DOCS.glitch.me/callbacks/sign_in_with_apple";
    // var clientID = "AS_PER_THE_DOCS";
       appleCredential = await SignInWithApple.getAppleIDCredential(
    scopes: [
          AppleIDAuthorizationScopes.email,
          AppleIDAuthorizationScopes.fullName,
          ],
        webAuthenticationOptions: WebAuthenticationOptions(
        clientId: clientID!,
          redirectUri: Uri.parse(
            redirectURL)));
    } else{


      final appleCredential = await SignInWithApple.getAppleIDCredential(
      scopes: [
        AppleIDAuthorizationScopes.email,
        AppleIDAuthorizationScopes.fullName,
      ],
        nonce: nonce,
      );
    }

      // Create an `OAuthCredential` from the credential returned by Apple.
      final oauthCredential = OAuthProvider("apple.com").credential(
        idToken: appleCredential.identityToken,
        accessToken: appleCredential.authorizationCode,
        rawNonce: rawNonce,
      );
      final fixDisplayNameFromApple = [
        appleCredential.givenName ?? '',
        appleCredential.familyName ?? '',
      ].join(' ').trim();

      await FirebaseAuth.instance.signInWithCredential(oauthCredential);
      ProgressDialog.hide();
      User? user = FirebaseAuth.instance.currentUser;

      if (fixDisplayNameFromApple != null &&
          fixDisplayNameFromApple != ' ' && user!=null &&
          user.displayName == null) {
        await user.updateDisplayName(fixDisplayNameFromApple);
        await user.reload();
      }
      return FirebaseAuth.instance.currentUser;

    } on FirebaseAuthException catch (e) {
      showToastMessage(e.message.toString());
      print('Firebase auth error');
      print(e);
      ProgressDialog.hide();
      return null;
    } catch (e) {
      ProgressDialog.hide();
      print(e.toString());
      return null;
      // AppHelper.showToastMessage(e.toString());
    }
  }

  String generateNonce([int length = 32]) {
    final charset =
        '0123456789ABCDEFGHIJKLMNOPQRSTUVXYZabcdefghijklmnopqrstuvwxyz-._';
    final random = Random.secure();
    return List.generate(length, (_) => charset[random.nextInt(charset.length)])
        .join();
  }

  /// Returns the sha256 hash of [input] in hex notation.
  String sha256ofString(String input) {
    final bytes = utf8.encode(input);
    final digest = sha256.convert(bytes);
    return digest.toString();
  }

  void showToastMessage(String message) {
    Fluttertoast.showToast(
        msg: message,
        toastLength: Toast.LENGTH_SHORT,
        gravity: ToastGravity.BOTTOM,
        timeInSecForIosWeb: 1,
        // backgroundColor: AppColor.darkBlueColor,
        // textColor: Colors.white,
        fontSize: 16.0);
  }
}